// Fichier Game.java
// Ce fichier contient la logique principale du jeu 2048. Il est indépendant de l'interface utilisateur.
/*
    Fonctions principales :
    - gameBoard : Matrice 2D (int[][]) représentant la grille du jeu.
    - score, highScore : Suivi du score et du meilleur score.
    - addNewNumbers() : Ajoute une nouvelle tuile (2, 4 ou 8) aléatoirement sur la grille.
    - pushUp(), pushDown(), pushLeft(), pushRight() : Logique de déplacement et de fusion des tuiles.
    - serialize(), deserialize() : Sauvegarde et restauration de l'état du jeu (grille, score, meilleur score) en chaînes de caractères.
    - loadHighScore(), saveHighScore(), loadGameState(), saveGameState() : Utilisation de SharedPreferences pour persister les données.
    - Constructeurs : Initialisation de la grille et chargement/restauration des données.

    Relations :
    - MainActivity : Crée une instance de Game et appelle ses méthodes pour la logique du jeu. MainActivity affiche l'état du jeu.
    - SharedPreferences : Game utilise SharedPreferences pour sauvegarder et charger le meilleur score et l'état du jeu.
*/

package legion.muyue.best2048;

import android.content.Context;
import android.content.SharedPreferences;

import androidx.annotation.NonNull;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class Game {

    private int[][] board; // Renommé
    private final Random randomNumberGenerator; // Renommé
    private int currentScore = 0; // Renommé
    private int highestScore = 0; // Renommé
    private static final int BOARD_SIZE = 4; // Ajout constante
    private static final String PREFS_NAME = "Best2048_Prefs"; // Ajout constante
    private static final String HIGH_SCORE_KEY = "high_score"; // Ajout constante
    private static final String GAME_STATE_KEY = "game_state"; // Ajout constante
    private final Context context; // Ajout contexte
    private boolean gameWon = false; // Ajout état
    private boolean gameOver = false; // Ajout état

    /**
     * Constructeur principal de la classe Game.
     *
     * @param context Le contexte Android (généralement une Activity).  Nécessaire pour accéder aux SharedPreferences.
     */
    public Game(Context context) {
        this.context = context;
        this.randomNumberGenerator = new Random();
        this.board = new int[BOARD_SIZE][BOARD_SIZE];
        loadHighScore();
        loadGameState(); // Charge l'état précédent ou initialise
    }

    /**
     * Constructeur pour la restauration d'une partie précédemment sauvegardée.
     *
     * @param board      Le plateau de jeu restauré (l'état des tuiles).
     * @param score      Le score au moment de la sauvegarde.
     * @param highScore  Le meilleur score enregistré au moment de la sauvegarde.
     * @param context    Le contexte Android, nécessaire pour les SharedPreferences.
     */
    public Game(int[][] board, int score, int highScore, Context context) {
        this.board = board;
        this.currentScore = score;
        this.highestScore = highScore;
        this.context = context;
        this.randomNumberGenerator = new Random();
    }

    /**
     * Récupère la valeur d'une cellule spécifique sur le plateau.
     *
     * @param row    L'index de la ligne (0 à BOARD_SIZE - 1).
     * @param column L'index de la colonne (0 à BOARD_SIZE - 1).
     * @return La valeur de la cellule à la position spécifiée.
     * @throws IllegalArgumentException Si row ou column sont en dehors des limites du plateau.
     */
    public int getCellValue(int row, int column) {
        if (row < 0 || row >= BOARD_SIZE || column < 0 || column >= BOARD_SIZE) {
            throw new IllegalArgumentException("Indices de ligne ou de colonne hors limites : row=" + row + ", column=" + column);
        }
        return this.board[row][column];
    }

    /**
     * Définit la valeur d'une cellule spécifique sur le plateau.
     *
     * @param row   L'index de la ligne (0 à BOARD_SIZE - 1).
     * @param col   L'index de la colonne (0 à BOARD_SIZE - 1).
     * @param value La nouvelle valeur de la cellule.
     * @throws IllegalArgumentException Si row ou col sont en dehors des limites du plateau.
     */
    public void setCellValue(int row, int col, int value) {
        if (row < 0 || row >= BOARD_SIZE || col < 0 || col >= BOARD_SIZE) {
            throw new IllegalArgumentException("Indices de ligne ou de colonne hors limites : row=" + row + ", col=" + col);
        }
        this.board[row][col] = value;
    }

    /**
     * Récupère le score actuel de la partie.
     *
     * @return Le score actuel.
     */
    public int getCurrentScore() {
        return this.currentScore;
    }

    /**
     * Met à jour le score actuel.
     *
     * @param currentScore Le nouveau score actuel.
     */
    public void setCurrentScore(int currentScore) {
        this.currentScore = currentScore;
    }

    /**
     * Récupère le meilleur score enregistré.
     *
     * @return Le meilleur score.
     */
    public int getHighestScore() {
        return this.highestScore;
    }

    /**
     * Met à jour le meilleur score.  Utilisé lors du chargement et après une partie.
     *
     * @param highestScore Le nouveau meilleur score.
     */
    public void setHighestScore(int highestScore) {
        this.highestScore = highestScore;
    }

    /**
     * Met à jour le board.  Utilisé lors du chargement.
     *
     * @param board Le nouveau board.
     */
    public void setBoard(int[][] board){
        this.board = board;
    }

    /**
     * Récupère le générateur de nombres aléatoires.
     *
     * @return Le générateur de nombres aléatoires.
     */
    public Random getRandomNumberGenerator() {
        return this.randomNumberGenerator;
    }

    /**
     * Indique si le joueur a gagné la partie (atteint une tuile de 2048).
     *
     * @return true si le joueur a gagné, false sinon.
     */
    public boolean isGameWon() {
        return this.gameWon;
    }

    /**
     * Indique si la partie est terminée (plus de mouvements possibles).
     *
     * @return true si la partie est terminée, false sinon.
     */
    public boolean isGameOver() {
        return this.gameOver;
    }

    /**
     * Définit l'état de la partie gagnée.
     *
     * @param gameWon true si le joueur a gagné, false sinon.
     */
    public void setGameWon(boolean gameWon) {
        this.gameWon = gameWon;
    }

    /**
     * Définit l'état de la partie terminée.
     *
     * @param gameOver true si la partie est terminée, false sinon.
     */
    public void setGameOver(boolean gameOver) {
        this.gameOver = gameOver;
    }

    /**
     * Charge le meilleur score à partir des préférences partagées (SharedPreferences).
     */
    public void loadHighScore() {
        SharedPreferences prefs = context.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
        setHighestScore(prefs.getInt(HIGH_SCORE_KEY, 0));
    }

    /**
     * Enregistre le meilleur score actuel dans les préférences partagées (SharedPreferences).
     */
    public void saveHighScore() {
        SharedPreferences prefs = context.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = prefs.edit();
        editor.putInt(HIGH_SCORE_KEY, getHighestScore());
        editor.apply();
    }

    /**
     * Charge l'état de la partie (plateau, score) à partir des préférences partagées.
     */
    private void loadGameState() {
        SharedPreferences prefs = context.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
        String savedState = prefs.getString(GAME_STATE_KEY, null);
        if (savedState != null) {
            Game savedGame = deserialize(savedState, context);
            if (savedGame != null) { // Vérifier si la désérialisation a réussi
                setBoard(savedGame.board);
                setCurrentScore(savedGame.currentScore);
                // Le meilleur score est déjà chargé par loadHighScore() appelé dans le constructeur principal
            } else {
                // Si l'état sauvegardé est invalide, commence une nouvelle partie
                initializeNewBoard();
            }
        } else {
            // Si aucun état n'est sauvegardé, commence une nouvelle partie
            initializeNewBoard();
        }
    }

    /**
     * Initialise le plateau pour une nouvelle partie (typiquement avec deux tuiles).
     */
    private void initializeNewBoard() {
        this.board = new int[BOARD_SIZE][BOARD_SIZE]; // Assure que le plateau est vide
        this.currentScore = 0;
        addNewTile();
        addNewTile();
    }


    /**
     * Enregistre l'état actuel de la partie (plateau, score) dans les préférences partagées.
     */
    public void saveGameState() {
        SharedPreferences prefs = context.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = prefs.edit();
        String serializedState = toString(); // Utilise la méthode toString() pour la sérialisation
        editor.putString(GAME_STATE_KEY, serializedState);
        editor.apply();
    }

    /**
     * Ajoute une nouvelle tuile aléatoire à une position aléatoire vide sur le plateau si possible.
     */
    public void addNewTile() { // Renommé
        if (!hasEmptyCell()) {
            return; // Ne fait rien si le plateau est plein
        }

        List<int[]> emptyCells = new ArrayList<>();

        // 1. Collecter les coordonnées des cellules vides.
        for (int row = 0; row < BOARD_SIZE; row++) {
            for (int col = 0; col < BOARD_SIZE; col++) {
                if (board[row][col] == 0) {
                    emptyCells.add(new int[]{row, col});
                }
            }
        }

        // 2. S'il y a des cellules vides, ajouter une nouvelle tuile.
        if (!emptyCells.isEmpty()) {
            int[] randomCell = emptyCells.get(getRandomNumberGenerator().nextInt(emptyCells.size()));
            int row = randomCell[0];
            int col = randomCell[1];

            int value = generateRandomTileValue(); // Utilise la nouvelle logique de probabilité
            setCellValue(row, col, value);
        }
    }

    /**
     * Génère une valeur de tuile aléatoire selon les probabilités définies.
     *
     * @return La valeur de la nouvelle tuile (2, 4, 8, ...) avec les probabilités spécifiées.
     */
    private int generateRandomTileValue() { // Logique de probabilité modifiée
        int randomValue = randomNumberGenerator.nextInt(10000);

        if (randomValue < 8540) { // 85.40%
            return 2;
        } else if (randomValue < 9740) { // 12.00%
            return 4;
        } else if (randomValue < 9940) { // 2.00%
            return 8;
        } else if (randomValue < 9990) { // 0.50%
            return 16;
        } else if (randomValue < 9995) { // 0.05%
            return 32;
        } else if (randomValue < 9998) { // 0.03%
            return 64;
        } else if (randomValue < 9999) { // 0.01%
            return 128;
        }else { // 0.01%
            return 256;
        }
    }

    /**
     * Déplace les tuiles vers le haut, en fusionnant les tuiles de même valeur.
     *
     * @return True si le plateau a été modifié, False sinon.
     */
    public boolean pushUp() { // Logique refactorisée, retourne boolean, utilise hasMerged
        boolean boardChanged = false;
        boolean[] hasMerged = new boolean[BOARD_SIZE];

        for (int col = 0; col < BOARD_SIZE; col++) {
            hasMerged = new boolean[BOARD_SIZE]; // Réinitialise par colonne
            for (int row = 1; row < BOARD_SIZE; row++) {
                if (getCellValue(row, col) != 0) {
                    int currentValue = getCellValue(row, col);
                    int currentRow = row;

                    // 1. Déplacer la tuile
                    while (currentRow > 0 && getCellValue(currentRow - 1, col) == 0) {
                        setCellValue(currentRow - 1, col, currentValue);
                        setCellValue(currentRow, col, 0);
                        currentRow--;
                        boardChanged = true;
                    }

                    // 2. Fusionner si possible
                    if (currentRow > 0 && getCellValue(currentRow - 1, col) == currentValue && !hasMerged[currentRow - 1]) {
                        int newValue = getCellValue(currentRow - 1, col) * 2;
                        setCellValue(currentRow - 1, col, newValue);
                        setCellValue(currentRow, col, 0); // La tuile d'origine disparaît
                        setCurrentScore(getCurrentScore() + newValue);
                        hasMerged[currentRow - 1] = true;
                        boardChanged = true;
                        updateHighestScore(); // Met à jour le meilleur score si nécessaire
                        // checkWinCondition(); // Vérifie si la victoire est atteinte
                    }
                }
            }
        }
        checkWinCondition(); // Vérifie après tous les mouvements de la colonne
        checkGameOverCondition(); // Vérifie si la partie est terminée
        return boardChanged;
    }


    /**
     * Déplace les tuiles vers le bas, en fusionnant les tuiles de même valeur.
     *
     * @return True si le plateau a été modifié, False sinon.
     */
    public boolean pushDown() { // Logique refactorisée
        boolean boardChanged = false;
        boolean[] hasMerged = new boolean[BOARD_SIZE];

        for (int col = 0; col < BOARD_SIZE; col++) {
            hasMerged = new boolean[BOARD_SIZE];
            for (int row = BOARD_SIZE - 2; row >= 0; row--) { // Itère de bas en haut
                if (getCellValue(row, col) != 0) {
                    int currentValue = getCellValue(row, col);
                    int currentRow = row;

                    // Déplacer vers le bas
                    while (currentRow < BOARD_SIZE - 1 && getCellValue(currentRow + 1, col) == 0) {
                        setCellValue(currentRow + 1, col, currentValue);
                        setCellValue(currentRow, col, 0);
                        currentRow++;
                        boardChanged = true;
                    }

                    // Fusionner si possible
                    if (currentRow < BOARD_SIZE - 1 && getCellValue(currentRow + 1, col) == currentValue && !hasMerged[currentRow + 1]) {
                        int newValue = getCellValue(currentRow + 1, col) * 2;
                        setCellValue(currentRow + 1, col, newValue);
                        setCellValue(currentRow, col, 0);
                        setCurrentScore(getCurrentScore() + newValue);
                        hasMerged[currentRow + 1] = true;
                        boardChanged = true;
                        updateHighestScore();
                        // checkWinCondition();
                    }
                }
            }
        }
        checkWinCondition();
        checkGameOverCondition();
        return boardChanged;
    }

    /**
     * Déplace les tuiles vers la gauche, en fusionnant les tuiles de même valeur.
     *
     * @return True si le plateau a été modifié, False sinon.
     */
    public boolean pushLeft() { // Logique refactorisée
        boolean boardChanged = false;
        boolean[] hasMerged = new boolean[BOARD_SIZE]; // Par ligne cette fois

        for (int row = 0; row < BOARD_SIZE; row++) {
            hasMerged = new boolean[BOARD_SIZE]; // Réinitialise par ligne
            for (int col = 1; col < BOARD_SIZE; col++) { // Commence à la 2ème colonne
                if (getCellValue(row, col) != 0) {
                    int currentValue = getCellValue(row, col);
                    int currentCol = col;

                    // Déplacer vers la gauche
                    while (currentCol > 0 && getCellValue(row, currentCol - 1) == 0) {
                        setCellValue(row, currentCol - 1, currentValue);
                        setCellValue(row, currentCol, 0);
                        currentCol--;
                        boardChanged = true;
                    }

                    // Fusionner si possible
                    if (currentCol > 0 && getCellValue(row, currentCol - 1) == currentValue && !hasMerged[currentCol - 1]) {
                        int newValue = getCellValue(row, currentCol - 1) * 2;
                        setCellValue(row, currentCol - 1, newValue);
                        setCellValue(row, currentCol, 0);
                        setCurrentScore(getCurrentScore() + newValue);
                        hasMerged[currentCol - 1] = true;
                        boardChanged = true;
                        updateHighestScore();
                        // checkWinCondition();
                    }
                }
            }
        }
        checkWinCondition();
        checkGameOverCondition();
        return boardChanged;
    }

    /**
     * Déplace les tuiles vers la droite, en fusionnant les tuiles de même valeur.
     *
     * @return True si le plateau a été modifié, False sinon.
     */
    public boolean pushRight() { // Logique refactorisée
        boolean boardChanged = false;
        boolean[] hasMerged = new boolean[BOARD_SIZE]; // Par ligne

        for (int row = 0; row < BOARD_SIZE; row++) {
            hasMerged = new boolean[BOARD_SIZE];
            for (int col = BOARD_SIZE - 2; col >= 0; col--) { // Itère de droite à gauche
                if (getCellValue(row, col) != 0) {
                    int currentValue = getCellValue(row, col);
                    int currentCol = col;

                    // Déplacer vers la droite
                    while (currentCol < BOARD_SIZE - 1 && getCellValue(row, currentCol + 1) == 0) {
                        setCellValue(row, currentCol + 1, currentValue);
                        setCellValue(row, currentCol, 0);
                        currentCol++;
                        boardChanged = true;
                    }

                    // Fusionner si possible
                    if (currentCol < BOARD_SIZE - 1 && getCellValue(row, currentCol + 1) == currentValue && !hasMerged[currentCol + 1]) {
                        int newValue = getCellValue(row, currentCol + 1) * 2;
                        setCellValue(row, currentCol + 1, newValue);
                        setCellValue(row, currentCol, 0);
                        setCurrentScore(getCurrentScore() + newValue);
                        hasMerged[currentCol + 1] = true;
                        boardChanged = true;
                        updateHighestScore();
                        // checkWinCondition();
                    }
                }
            }
        }
        checkWinCondition();
        checkGameOverCondition();
        return boardChanged;
    }


    /**
     * Met à jour le meilleur score si le score actuel le dépasse, et sauvegarde le nouveau meilleur score.
     */
    private void updateHighestScore() { // Renommé
        if (getCurrentScore() > getHighestScore()) {
            setHighestScore(getCurrentScore());
            saveHighScore(); // Sauvegarde implémentée
        }
    }

    /**
     * Sérialise l'état du jeu (plateau, score courant, meilleur score) en une chaîne de caractères.
     * Format: "row1col1,row1col2,...,row1colN,row2col1,...,rowNcolN,currentScore,highestScore"
     *
     * @return Une chaîne représentant l'état complet du jeu.
     */
    @NonNull
    @Override
    public String toString() { // Renommé et Override
        StringBuilder sb = new StringBuilder();

        for (int row = 0; row < BOARD_SIZE; row++) {
            for (int col = 0; col < BOARD_SIZE; col++) {
                sb.append(board[row][col]).append(","); // Utilise board directement
            }
        }

        sb.append(getCurrentScore()).append(",");
        sb.append(getHighestScore()); // Sérialise highestScore

        return sb.toString();
    }


    /**
     * Désérialise une chaîne de caractères pour restaurer l'état du jeu.
     *
     * @param serializedState La chaîne représentant l'état du jeu (au format de la méthode toString).
     * @param context         Le contexte Android (nécessaire pour le constructeur de Game).
     * @return Un nouvel objet Game restauré, ou null si la désérialisation échoue.
     */
    public static Game deserialize(String serializedState, Context context) { // Logique améliorée
        if (serializedState == null || serializedState.isEmpty()) {
            return null;
        }

        String[] values = serializedState.split(",");
        // Vérifie si le nombre d'éléments correspond à la taille attendue (board + score + highScore)
        if (values.length != (BOARD_SIZE * BOARD_SIZE + 2)) {
            System.err.println("Erreur de désérialisation : nombre d'éléments incorrect. Attendu=" + (BOARD_SIZE * BOARD_SIZE + 2) + ", Obtenu=" + values.length);
            return null; // Longueur incorrecte
        }

        int[][] newBoard = new int[BOARD_SIZE][BOARD_SIZE];
        int index = 0;

        try {
            for (int row = 0; row < BOARD_SIZE; row++) {
                for (int col = 0; col < BOARD_SIZE; col++) {
                    newBoard[row][col] = Integer.parseInt(values[index++]);
                }
            }

            int score = Integer.parseInt(values[index++]);
            int highScore = Integer.parseInt(values[index++]); // Désérialise highScore

            // Utilise le constructeur qui prend le contexte
            return new Game(newBoard, score, highScore, context);

        } catch (NumberFormatException | ArrayIndexOutOfBoundsException e) {
            System.err.println("Erreur de désérialisation : " + e.getMessage());
            // e.printStackTrace(); // Optionnel: pour plus de détails dans Logcat
            return null; // Erreur de format ou index hors limites
        }
    }


    /**
     * Vérifie si la condition de victoire (tuile 2048) est remplie.
     * Met à jour la variable gameWon.
     */
    private void checkWinCondition() { // Ajouté
        // Ne vérifie que si la partie n'est pas déjà gagnée
        if (!isGameWon()) {
            for(int row = 0 ; row < BOARD_SIZE; row++){
                for (int col = 0; col < BOARD_SIZE; col++){
                    if(getCellValue(row, col) >= 2048){ // Condition >= 2048
                        setGameWon(true);
                        // Optionnel : On pourrait arrêter la vérification ici
                        // return;
                    }
                }
            }
        }
    }

    /**
     * Vérifie si la condition de fin de partie (aucun mouvement possible) est remplie.
     * Met à jour la variable gameOver.
     */
    private void checkGameOverCondition() { // Ajouté
        // Si une case est vide, la partie n'est pas terminée
        if (hasEmptyCell()) {
            setGameOver(false);
            return;
        }

        // Vérifie les fusions possibles horizontalement et verticalement
        for (int row = 0; row < BOARD_SIZE; row++) {
            for (int col = 0; col < BOARD_SIZE; col++) {
                int currentValue = getCellValue(row, col);

                // Vérifie voisin du haut
                if (row > 0 && getCellValue(row - 1, col) == currentValue) {
                    setGameOver(false);
                    return;
                }
                // Vérifie voisin du bas
                if (row < BOARD_SIZE - 1 && getCellValue(row + 1, col) == currentValue) {
                    setGameOver(false);
                    return;
                }
                // Vérifie voisin de gauche
                if (col > 0 && getCellValue(row, col - 1) == currentValue) {
                    setGameOver(false);
                    return;
                }
                // Vérifie voisin de droite
                if (col < BOARD_SIZE - 1 && getCellValue(row, col + 1) == currentValue) {
                    setGameOver(false);
                    return;
                }
            }
        }

        // Si aucune case vide et aucune fusion possible, la partie est terminée
        setGameOver(true);
    }

    /**
     * Vérifie s'il existe au moins une cellule vide sur le plateau.
     *
     * @return true s'il y a une cellule vide, false sinon.
     */
    private boolean hasEmptyCell() { // Ajouté
        for (int row = 0; row < BOARD_SIZE; row++) {
            for (int col = 0; col < BOARD_SIZE; col++) {
                if (getCellValue(row, col) == 0) {
                    return true;
                }
            }
        }
        return false;
    }

}