// Fichier MainActivity.java
// C'est l'activité principale de l'application, l'écran principal avec lequel l'utilisateur interagit.
/*
    Fonctions principales :
    - Gère l'interface utilisateur :
        - Initialise le GridLayout (gameBoardLayout).
        - Met à jour les TextView pour le score (scoreTextView) et le meilleur score (highScoreTextView).
        - updateUI() : Redessine la grille en fonction de l'état de l'objet Game.
        - setTileAppearance() : Applique le style visuel (couleur, taille du texte) aux tuiles.
        - Gère les entrées utilisateur (swipes) via OnSwipeTouchListener.
        - Gère les clics sur les boutons (Recommencer, Statistiques, Menu, Multijoueur).
        - Affiche des boîtes de dialogue (confirmation de redémarrage, statistiques).
    - Gère le cycle de vie de l'activité :
        - onCreate() : Initialisation.
        - onResume() : Reprise de l'application.
        - onPause() : Sauvegarde de l'état et des statistiques.
    - Gère la persistance des données (SharedPreferences) : état du jeu, meilleur score, statistiques.
    - showStats(), hideStats() : Gère l'affichage/masquage des statistiques (ViewStub).
    - Gère les statistiques du jeu.

    Relations :
    - Game : Interagit constamment avec l'objet Game.
    - OnSwipeTouchListener : Détection des swipes.
    - res/layout/*.xml : Utilise les fichiers de layout pour l'interface utilisateur.
    - res/drawable/*.xml : Apparence des tuiles et boutons.
    - res/values/*.xml : Couleurs, dimensions, chaînes, styles.
    - res/anim/*.xml : Animation du bouton multijoueur.
    - SharedPreferences : Sauvegarde et chargement des données.
*/

package legion.muyue.best2048;

import android.annotation.SuppressLint; // Ajout pour ClickableViewAccessibility
import android.app.AlertDialog;
import android.content.Context; // Non utilisé directement mais ok
import android.content.SharedPreferences;
import android.os.Bundle;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewStub; // Ajout pour ViewStub
import android.view.animation.AnimationUtils;
import android.widget.TextView;

import androidx.activity.EdgeToEdge;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.ContextCompat;
import androidx.gridlayout.widget.GridLayout;
import android.widget.Button;


public class MainActivity extends AppCompatActivity {

    // UI Elements (renommés/ajoutés)
    private GridLayout boardGridLayout;
    private TextView currentScoreTextView;
    private TextView highestScoreTextView;
    private Button newGameButton;
    private Button multiplayerButton;
    private Button statisticsButton;
    private Button menuButton;
    private ViewStub statisticsViewStub; // Ajout pour stats

    // Game Logic
    private Game game;
    private static final int BOARD_SIZE = 4; // Ajout constante

    // Preferences
    private SharedPreferences preferences; // Renommé
    // Constantes déjà définies dans Game, redéfinies ici ? Ok pour l'instant.
    private static final String PREFS_NAME = "Best2048_Prefs";
    private static final String HIGH_SCORE_KEY = "high_score";
    private static final String GAME_STATE_KEY = "game_state";

    // Statistics (nouveaux champs)
    private boolean statisticsVisible = false;
    private int totalGamesPlayed = 0;
    private int totalGamesStarted = 0;
    private int totalMoves = 0;
    private int currentMoves = 0;
    private long totalPlayTimeMs = 0;
    private long currentGameStartTimeMs = 0;
    private int mergesThisGame = 0;
    private int totalMerges = 0;
    private int highestTile = 0;
    private int numberOfTimesObjectiveReached = 0;
    private int perfectGames = 0;
    private long averageGameTimeMs = 0;
    private long bestWinningTimeMs = Long.MAX_VALUE;
    private long worstWinningTimeMs = 0;

    // Multiplayer Statistics (nouveaux champs)
    private int multiplayerGamesWon = 0;
    private int multiplayerGamesPlayed = 0;
    private int multiplayerBestWinningStreak = 0;
    private int multiplayerAverageScore = 0;
    private long multiplayerAverageGameTimeMs = 0;
    private int totalMultiplayerLosses = 0;
    private int multiplayerHighestScore = 0;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        EdgeToEdge.enable(this);
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        findViews(); // Refactorisation onCreate
        initializeGame(); // Refactorisation onCreate
        setupListeners(); // Refactorisation onCreate
    }

    // Getters/Setters ajoutés (pour UI et Game)
    public void setBoardGridLayout(GridLayout boardGridLayout) { this.boardGridLayout = boardGridLayout; }
    public void setCurrentScoreTextView(TextView currentScoreTextView) { this.currentScoreTextView = currentScoreTextView; }
    public void setHighestScoreTextView(TextView highestScoreTextView) { this.highestScoreTextView = highestScoreTextView; }
    public void setNewGameButton(Button newGameButton) { this.newGameButton = newGameButton; }
    public void setMultiplayerButton(Button multiplayerButton) { this.multiplayerButton = multiplayerButton; }
    public void setStatisticsButton(Button statisticsButton) { this.statisticsButton = statisticsButton; }
    public void setMenuButton(Button menuButton) { this.menuButton = menuButton; }
    public void setStatisticsViewStub(ViewStub statisticsViewStub) { this.statisticsViewStub = statisticsViewStub; }
    public void setPreferences(SharedPreferences preferences) { this.preferences = preferences; }
    public void setGame(Game game) { this.game = game; }
    public Button getNewGameButton() { return this.newGameButton; }
    public Button getMultiplayerButton() { return this.multiplayerButton; }
    public Button getStatisticsButton() { return this.statisticsButton; }
    public Button getMenuButton() { return this.menuButton; }
    public GridLayout getBoardGridLayout() { return this.boardGridLayout; }
    public Game getGame() { return this.game; }
    public TextView getCurrentScoreTextView() { return this.currentScoreTextView; }
    public TextView getHighestScoreTextView() { return this.highestScoreTextView; }

    /**
     * Initialise les références aux éléments de l'interface utilisateur (UI).
     */
    private void findViews() { // Nouvelle méthode
        setBoardGridLayout(findViewById(R.id.gameBoard));
        setCurrentScoreTextView(findViewById(R.id.scoreLabel));
        setHighestScoreTextView(findViewById(R.id.highScoreLabel));
        setNewGameButton(findViewById(R.id.restartButton)); // ID reste restartButton
        setStatisticsButton(findViewById(R.id.statsButton)); // ID reste statsButton
        setMenuButton(findViewById(R.id.menuButton)); // ID reste menuButton
        setMultiplayerButton(findViewById(R.id.multiplayerButton));
        setStatisticsViewStub(findViewById(R.id.statsViewStub)); // Ajout ViewStub
    }

    /**
     * Initialise l'objet Game, charge le meilleur score et restaure l'état de la partie (si disponible).
     */
    private void initializeGame() { // Nouvelle méthode
        setPreferences(getSharedPreferences(PREFS_NAME, MODE_PRIVATE));
        setGame(new Game(this)); // Passe le contexte à Game
        // loadGame(); // Est appelé implicitement par le constructeur de Game via loadGameState
        updateUI(); // MAJ initiale de l'UI
        loadStatistics(); // Charge les stats
        // Initialise le timer pour la partie chargée ou nouvelle
        currentGameStartTimeMs = System.currentTimeMillis(); // Démarre timer ici
        totalGamesStarted++; // Incrémente car une partie commence (nouvelle ou chargée)
    }

    /**
     * Configure les listeners pour les interactions de l'utilisateur (boutons, swipes).
     */
    private void setupListeners() { // Nouvelle méthode (contenu déplacé)
        getNewGameButton().setOnClickListener(v -> {
            v.startAnimation(AnimationUtils.loadAnimation(this, R.anim.button_press));
            // Appelle maintenant showRestartConfirmationDialog au lieu de startNewGame directement
            showRestartConfirmationDialog();
        });

        getStatisticsButton().setOnClickListener(v -> {
            v.startAnimation(AnimationUtils.loadAnimation(this, R.anim.button_press));
            showStatistics(); // Appelle la nouvelle méthode pour les stats
        });

        getMenuButton().setOnClickListener(v -> {
            v.startAnimation(AnimationUtils.loadAnimation(this, R.anim.button_press));
            showMenu(); // Appelle le stub de menu
        });

        getMultiplayerButton().setOnClickListener(v -> {
            v.startAnimation(AnimationUtils.loadAnimation(this, R.anim.button_press));
            showMultiplayerScreen(); // Appelle le stub multijoueur
        });

        setupSwipeListener(); // Configure le swipe listener
    }

    /**
     * Initialise le GridLayout qui représente le plateau de jeu. Définit le nombre de lignes et colonnes.
     */
    private void initGameBoardLayout() { // Inchangé mais utilise getter
        getBoardGridLayout().removeAllViews();
        getBoardGridLayout().setColumnCount(BOARD_SIZE);
        getBoardGridLayout().setRowCount(BOARD_SIZE);
    }

    /**
     * Met à jour l'ensemble de l'interface utilisateur : le plateau, le score actuel et le meilleur score.
     */
    private void updateUI() { // Refactorisé
        updateBoard();
        updateScores();
    }

    /**
     * Met à jour l'affichage des tuiles sur le plateau de jeu.
     */
    private void updateBoard() { // Nouvelle méthode (contenu de l'ancien updateUI partie grille)
        getBoardGridLayout().removeAllViews();

        for (int row = 0; row < BOARD_SIZE; row++) {
            for (int col = 0; col < BOARD_SIZE; col++) {
                TextView tileTextView = new TextView(this);
                int value = getGame().getCellValue(row, col); // Utilise getter renommé

                setTileStyle(tileTextView, value); // Utilise méthode renommée

                GridLayout.LayoutParams params = new GridLayout.LayoutParams();
                params.width = 0;
                params.height = 0;
                params.rowSpec = GridLayout.spec(row, 1f);
                params.columnSpec = GridLayout.spec(col, 1f);

                int margin = (int) getResources().getDimension(R.dimen.tile_margin);
                params.setMargins(margin, margin, margin, margin);

                tileTextView.setLayoutParams(params);

                getBoardGridLayout().addView(tileTextView);
            }
        }
    }

    /**
     * Met à jour les TextViews du score actuel et du meilleur score.
     */
    private void updateScores() { // Nouvelle méthode (contenu de l'ancien updateUI partie score)
        getCurrentScoreTextView().setText(getString(R.string.score_placeholder, getGame().getCurrentScore()));
        getHighestScoreTextView().setText(getString(R.string.high_score_placeholder, getGame().getHighestScore()));
    }


    /**
     * Définit l'apparence d'une tuile (couleur de fond, couleur du texte, taille du texte)
     * en fonction de sa valeur. Utilise un seul Drawable avec des teintes de couleur.
     *
     * @param tileTextView La TextView représentant la tuile.
     * @param value        La valeur de la tuile (0, 2, 4, 8, ...).
     */
    private void setTileStyle(TextView tileTextView, int value) { // Renommé et logique couleur texte corrigée

        tileTextView.setText(value > 0 ? String.valueOf(value) : "");
        tileTextView.setGravity(Gravity.CENTER);
        tileTextView.setTypeface(null, android.graphics.Typeface.BOLD); // Ajout police grasse

        int backgroundColorId;
        int textColorId;
        int textSizeId;

        switch (value) {
            case 0:
                backgroundColorId = R.color.tile_empty;
                textColorId = android.R.color.transparent;
                textSizeId = R.dimen.text_size_tile_small;
                break;
            case 2:
                backgroundColorId = R.color.tile_2;
                textColorId = R.color.text_tile_low;
                textSizeId = R.dimen.text_size_tile_small;
                break;
            case 4:
                backgroundColorId = R.color.tile_4;
                textColorId = R.color.text_tile_low;
                textSizeId = R.dimen.text_size_tile_small;
                break;
            case 8:
                backgroundColorId = R.color.tile_8;
                textColorId = R.color.text_tile_high; // Correction couleur texte
                textSizeId = R.dimen.text_size_tile_small;
                break;
            case 16:
                backgroundColorId = R.color.tile_16;
                textColorId = R.color.text_tile_high; // Correction
                textSizeId = R.dimen.text_size_tile_small;
                break;
            case 32:
                backgroundColorId = R.color.tile_32;
                textColorId = R.color.text_tile_high; // Correction
                textSizeId = R.dimen.text_size_tile_small;
                break;
            case 64:
                backgroundColorId = R.color.tile_64;
                textColorId = R.color.text_tile_high; // Correction
                textSizeId = R.dimen.text_size_tile_small;
                break;
            case 128:
                backgroundColorId = R.color.tile_128;
                textColorId = R.color.text_tile_high;
                textSizeId = R.dimen.text_size_tile_medium; // Changement taille
                break;
            case 256:
                backgroundColorId = R.color.tile_256;
                textColorId = R.color.text_tile_high;
                textSizeId = R.dimen.text_size_tile_medium;
                break;
            case 512:
                backgroundColorId = R.color.tile_512;
                textColorId = R.color.text_tile_high;
                textSizeId = R.dimen.text_size_tile_medium;
                break;
            case 1024:
                backgroundColorId = R.color.tile_1024;
                textColorId = R.color.text_tile_high;
                textSizeId = R.dimen.text_size_tile_large; // Changement taille
                break;
            case 2048:
                backgroundColorId = R.color.tile_2048;
                textColorId = R.color.text_tile_high;
                textSizeId = R.dimen.text_size_tile_large;
                break;
            default: // > 2048
                backgroundColorId = R.color.tile_super;
                textColorId = R.color.text_tile_high;
                textSizeId = R.dimen.text_size_tile_large;
                break;
        }

        tileTextView.setBackgroundResource(R.drawable.tile_background); // Utilise drawable unique
        tileTextView.getBackground().setTint(ContextCompat.getColor(this, backgroundColorId)); // Applique teinte

        tileTextView.setTextColor(ContextCompat.getColor(this, textColorId));
        tileTextView.setTextSize(TypedValue.COMPLEX_UNIT_PX, getResources().getDimension(textSizeId));
    }


    /**
     * Configure le listener de swipe sur le GridLayout pour gérer les mouvements des tuiles.
     */
    @SuppressLint("ClickableViewAccessibility") // Ajout annotation
    private void setupSwipeListener() { // Inchangé mais utilise getter
        getBoardGridLayout().setOnTouchListener(new OnSwipeTouchListener(this, new OnSwipeTouchListener.SwipeListener() {
            @Override public void onSwipeTop() { handleSwipe(Direction.UP); }
            @Override public void onSwipeBottom() { handleSwipe(Direction.DOWN); }
            @Override public void onSwipeLeft() { handleSwipe(Direction.LEFT); }
            @Override public void onSwipeRight() { handleSwipe(Direction.RIGHT); }
        }));
    }

    /**
     * Gère un swipe dans une direction donnée, en appelant la méthode appropriée de la classe Game
     * et en mettant à jour l'interface utilisateur.
     *
     * @param direction La direction du swipe (UP, DOWN, LEFT, RIGHT).
     */
    private void handleSwipe(Direction direction) { // Logique modifiée pour stats et win/loss
        boolean boardChanged; // Vérifie si le plateau a changé
        switch (direction) {
            case UP:    boardChanged = getGame().pushUp();    break;
            case DOWN:  boardChanged = getGame().pushDown();  break;
            case LEFT:  boardChanged = getGame().pushLeft();  break;
            case RIGHT: boardChanged = getGame().pushRight(); break;
            default:    boardChanged = false;
        }

        if (boardChanged) {
            currentMoves++; // Stat
            totalMoves++;   // Stat
            // mergesThisGame++; // Fusion est comptée dans Game ou ici? (pas dans ce snippet)
            totalMerges++; // Approximation, devrait être compté par fusion réelle

            // Trouve la tuile la plus haute pour les stats
            int currentHighest = 0;
            for (int row = 0; row < BOARD_SIZE; row++) {
                for (int col = 0; col < BOARD_SIZE; col++) {
                    if (game.getCellValue(row, col) > currentHighest) {
                        currentHighest = game.getCellValue(row, col);
                    }
                }
            }
            if (currentHighest > highestTile) {
                highestTile = currentHighest; // Met à jour stat highestTile
            }


            getGame().addNewTile(); // Ajoute une nouvelle tuile
            updateUI(); // Met à jour l'affichage

            // Vérifie victoire ou défaite
            if (getGame().isGameWon()) {
                // Gérer la victoire (par exemple, afficher un message)
                if(game.getCellValue(0,0) == 2048){ //Condition spécifique du snippet ? semble être un placeholder
                    numberOfTimesObjectiveReached++; // Stat
                    // Gérer les temps de victoire (best/worst)
                    long timeTaken = System.currentTimeMillis() - currentGameStartTimeMs;
                    if(timeTaken < bestWinningTimeMs){
                        bestWinningTimeMs = timeTaken;
                    }
                    if(timeTaken > worstWinningTimeMs){
                        worstWinningTimeMs = timeTaken;
                    }
                }
                showGameWonDialog(); // Affiche dialog victoire
            } else if (getGame().isGameOver()) {
                // Gérer la défaite
                totalGamesPlayed++; // Stat
                showGameOverDialog(); // Affiche dialog défaite
            }

        }
    }


    /**
     * Énumération représentant les directions possibles de swipe.
     */
    enum Direction { UP, DOWN, LEFT, RIGHT }

    /**
     * Affiche une boîte de dialogue de confirmation avant de recommencer la partie.
     * Utilise un layout XML personnalisé (dialog_restart_confirm.xml).
     */
    private void showRestartConfirmationDialog() { // Inchangé mais appelle startNewGame
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        LayoutInflater inflater = getLayoutInflater();
        View dialogView = inflater.inflate(R.layout.dialog_restart_confirm, null);
        builder.setView(dialogView);

        TextView dialogTitle = dialogView.findViewById(R.id.dialogTitle);
        TextView dialogMessage = dialogView.findViewById(R.id.dialogMessage);
        Button cancelButton = dialogView.findViewById(R.id.dialogCancelButton);
        Button confirmButton = dialogView.findViewById(R.id.dialogConfirmButton);

        final AlertDialog dialog = builder.create();

        cancelButton.setOnClickListener(v -> dialog.dismiss());

        confirmButton.setOnClickListener(v -> {
            dialog.dismiss();
            startNewGame(); // Appelle la nouvelle méthode
        });

        dialog.show();
    }

    /**
     * Commence une nouvelle partie : réinitialise le jeu et les statistiques de la partie en cours.
     */
    private void startNewGame() { // Nouvelle méthode (anciennement restartGame) + reset stats
        totalGamesStarted++; // Stat
        currentMoves = 0; // Reset stat partie
        mergesThisGame = 0; // Reset stat partie
        currentGameStartTimeMs = System.currentTimeMillis(); // Reset timer partie
        setGame(new Game(this)); // Crée un nouveau jeu
        // game.addNewTile(); // Le constructeur de Game s'en charge maintenant
        // game.addNewTile();
        updateUI(); // Met à jour l'affichage
    }


    /**
     * Affiche les statistiques du jeu en utilisant un ViewStub pour le chargement différé.
     */
    private void showStatistics() { // Nouvelle méthode
        statisticsVisible = !statisticsVisible;

        if (statisticsVisible) {
            if (statisticsViewStub.getParent() != null) {
                statisticsViewStub.inflate();
            }
            updateStatisticsTextViews(); // Met à jour les textes des stats
            findViewById(R.id.statistics_layout).setVisibility(View.VISIBLE); // Rend visible le layout gonflé
            getMultiplayerButton().setVisibility(View.GONE); // Cache bouton multi
        } else {
            hideStatistics(); // Cache si on reclique
        }
    }

    /**
     * Met à jour tous les TextViews à l'intérieur du layout des statistiques (qui doit déjà être gonflé).
     */
    private void updateStatisticsTextViews() { // Nouvelle méthode
        View inflatedStatsView = findViewById(R.id.statistics_layout); // Récupère la vue gonflée
        if (inflatedStatsView != null) { // Vérifie si la vue existe
            TextView highScoreStatsLabel = inflatedStatsView.findViewById(R.id.high_score_stats_label);
            TextView totalGamesPlayedLabel = inflatedStatsView.findViewById(R.id.total_games_played_label);
            TextView totalGamesStartedLabel = inflatedStatsView.findViewById(R.id.total_games_started_label);
            TextView winPercentageLabel = inflatedStatsView.findViewById(R.id.win_percentage_label);
            TextView totalPlayTimeLabel = inflatedStatsView.findViewById(R.id.total_play_time_label);
            TextView totalMovesLabel = inflatedStatsView.findViewById(R.id.total_moves_label);
            TextView currentMovesLabel = inflatedStatsView.findViewById(R.id.current_moves_label);
            TextView currentGameTimeLabel = inflatedStatsView.findViewById(R.id.current_game_time_label);
            TextView averageGameTimeLabel = inflatedStatsView.findViewById(R.id.average_game_time_label); // Pour solo
            TextView bestWinningTimeLabel = inflatedStatsView.findViewById(R.id.best_winning_time_label);
            TextView worstWinningTimeLabel = inflatedStatsView.findViewById(R.id.worst_winning_time_label);
            TextView totalMergesLabel = inflatedStatsView.findViewById(R.id.total_merges_label);
            TextView highestTileLabel = inflatedStatsView.findViewById(R.id.highest_tile_label);
            TextView objectiveReachedLabel = inflatedStatsView.findViewById(R.id.number_of_time_objective_reached_label);
            TextView perfectGameLabel = inflatedStatsView.findViewById(R.id.perfect_game_label);
            TextView multiplayerGamesWonLabel = inflatedStatsView.findViewById(R.id.multiplayer_games_won_label);
            TextView multiplayerGamesPlayedLabel = inflatedStatsView.findViewById(R.id.multiplayer_games_played_label);
            TextView multiplayerWinRateLabel = inflatedStatsView.findViewById(R.id.multiplayer_win_rate_label);
            TextView multiplayerBestWinningStreakLabel = inflatedStatsView.findViewById(R.id.multiplayer_best_winning_streak_label);
            TextView multiplayerAverageScoreLabel = inflatedStatsView.findViewById(R.id.multiplayer_average_score_label);
            TextView averageTimePerGameMultiLabel = inflatedStatsView.findViewById(R.id.average_time_per_game_label); // ID dupliqué? Utilisons le 2eme
            TextView totalMultiplayerLossesLabel = inflatedStatsView.findViewById(R.id.total_multiplayer_losses_label);
            TextView multiplayerHighScoreLabel = inflatedStatsView.findViewById(R.id.multiplayer_high_score_label);
            TextView mergesThisGameLabel = inflatedStatsView.findViewById(R.id.merges_this_game);

            // MAJ Textes
            highScoreStatsLabel.setText(getString(R.string.high_score_stats, game.getHighestScore()));
            totalGamesPlayedLabel.setText(getString(R.string.total_games_played, totalGamesPlayed));
            totalGamesStartedLabel.setText(getString(R.string.total_games_started, totalGamesStarted));
            totalMovesLabel.setText(getString(R.string.total_moves, totalMoves));
            currentMovesLabel.setText(getString(R.string.current_moves, currentMoves));
            mergesThisGameLabel.setText(getString(R.string.merges_this_game_label, mergesThisGame));
            totalMergesLabel.setText(getString(R.string.total_merges, totalMerges));
            highestTileLabel.setText(getString(R.string.highest_tile, highestTile));
            objectiveReachedLabel.setText(getString(R.string.number_of_time_objective_reached, numberOfTimesObjectiveReached));
            perfectGameLabel.setText(getString(R.string.perfect_games, perfectGames)); // Il manque la stat perfectGames dans les champs
            multiplayerGamesWonLabel.setText(getString(R.string.multiplayer_games_won, multiplayerGamesWon));
            multiplayerGamesPlayedLabel.setText(getString(R.string.multiplayer_games_played, multiplayerGamesPlayed));
            multiplayerBestWinningStreakLabel.setText(getString(R.string.multiplayer_best_winning_streak, multiplayerBestWinningStreak));
            multiplayerAverageScoreLabel.setText(getString(R.string.multiplayer_average_score, multiplayerGamesPlayed > 0 ? (totalMerges / multiplayerGamesPlayed) : 0)); // Calcul approximatif
            totalMultiplayerLossesLabel.setText(getString(R.string.total_multiplayer_losses, totalMultiplayerLosses));
            multiplayerHighScoreLabel.setText(getString(R.string.multiplayer_high_score, multiplayerHighestScore));

            // Calculs Pourcentages et Temps
            String winPercentage = (totalGamesStarted > 0) ? String.format("%.2f%%", ((double) numberOfTimesObjectiveReached / totalGamesStarted) * 100) : "N/A"; // Utilise numberOfTimesObjectiveReached
            winPercentageLabel.setText(getString(R.string.win_percentage, winPercentage));

            String multiplayerWinRate = (multiplayerGamesPlayed > 0) ? String.format("%.2f%%", ((double) multiplayerGamesWon / multiplayerGamesPlayed) * 100) : "N/A";
            multiplayerWinRateLabel.setText(getString(R.string.multiplayer_win_rate, multiplayerWinRate));

            totalPlayTimeLabel.setText(getString(R.string.total_play_time, formatTime(totalPlayTimeMs)));
            // Calcule le temps de la partie en cours dynamiquement si elle n'est pas finie
            long currentDuration = (game != null && !game.isGameOver() && !game.isGameWon()) ? System.currentTimeMillis() - currentGameStartTimeMs : 0;
            currentGameTimeLabel.setText(getString(R.string.current_game_time, formatTime(currentDuration)));

            long avgTimeSolo = (numberOfTimesObjectiveReached > 0) ? totalPlayTimeMs / numberOfTimesObjectiveReached : 0; // Basé sur victoires? Ou parties jouées?
            averageGameTimeLabel.setText(getString(R.string.average_time_per_game, formatTime(avgTimeSolo))); // ID 1

            long avgTimeMulti = (multiplayerGamesPlayed > 0) ? multiplayerAverageGameTimeMs / multiplayerGamesPlayed : 0;
            averageTimePerGameMultiLabel.setText(getString(R.string.average_time_per_game_label, formatTime(avgTimeMulti))); // ID 2

            bestWinningTimeLabel.setText(getString(R.string.best_winning_time, (bestWinningTimeMs != Long.MAX_VALUE) ? formatTime(bestWinningTimeMs) : "N/A"));
            worstWinningTimeLabel.setText(getString(R.string.worst_winning_time, (worstWinningTimeMs != 0) ? formatTime(worstWinningTimeMs) : "N/A"));

            // Listener bouton Back dans les stats
            Button backButton = inflatedStatsView.findViewById(R.id.backButton);
            backButton.setOnClickListener(v -> hideStatistics());
        }
    }

    /**
     * Masque la vue des statistiques et réaffiche le bouton multijoueur.
     */
    private void hideStatistics() { // Nouvelle méthode
        View inflatedStatsView = findViewById(R.id.statistics_layout);
        if (inflatedStatsView != null) {
            inflatedStatsView.setVisibility(View.GONE);
        }
        getMultiplayerButton().setVisibility(View.VISIBLE); // Réaffiche bouton multi
        statisticsVisible = false; // Met à jour l'état
    }

    /**
     * Charge les statistiques depuis les SharedPreferences.
     */
    private void loadStatistics() { // Nouvelle méthode
        totalGamesPlayed = preferences.getInt("totalGamesPlayed", 0);
        totalGamesStarted = preferences.getInt("totalGamesStarted", 0);
        totalMoves = preferences.getInt("totalMoves", 0);
        // currentMoves n'est pas chargé, il est spécifique à la session
        totalPlayTimeMs = preferences.getLong("totalPlayTimeMs", 0);
        // currentGameStartTimeMs est initialisé dans initializeGame ou repris
        // mergesThisGame n'est pas chargé
        totalMerges = preferences.getInt("totalMerges", 0);
        highestTile = preferences.getInt("highestTile", 0);
        numberOfTimesObjectiveReached = preferences.getInt("numberOfTimesObjectiveReached", 0);
        perfectGames = preferences.getInt("perfectGames", 0); // Chargement stat
        // averageGameTimeMs n'est pas chargé, recalculé
        bestWinningTimeMs = preferences.getLong("bestWinningTimeMs", Long.MAX_VALUE);
        worstWinningTimeMs = preferences.getLong("worstWinningTimeMs", 0);

        multiplayerGamesWon = preferences.getInt("multiplayerGamesWon", 0);
        multiplayerGamesPlayed = preferences.getInt("multiplayerGamesPlayed", 0);
        multiplayerBestWinningStreak = preferences.getInt("multiplayerBestWinningStreak", 0);
        // multiplayerAverageScore recalculé
        multiplayerAverageGameTimeMs = preferences.getLong("multiplayerAverageGameTimeMs", 0);
        totalMultiplayerLosses = preferences.getInt("totalMultiplayerLosses", 0);
        multiplayerHighestScore = preferences.getInt("multiplayerHighScore", 0);
    }

    /**
     * Sauvegarde les statistiques dans les SharedPreferences.
     */
    private void saveStatistics() { // Nouvelle méthode
        SharedPreferences.Editor editor = preferences.edit();
        editor.putInt("totalGamesPlayed", totalGamesPlayed);
        editor.putInt("totalGamesStarted", totalGamesStarted);
        editor.putInt("totalMoves", totalMoves);
        // currentMoves non sauvegardé
        editor.putLong("totalPlayTimeMs", totalPlayTimeMs);
        // currentGameStartTimeMs non sauvegardé directement, utilisé pour calculer totalPlayTimeMs
        // mergesThisGame non sauvegardé
        editor.putInt("totalMerges", totalMerges);
        editor.putInt("highestTile", highestTile);
        editor.putInt("numberOfTimesObjectiveReached", numberOfTimesObjectiveReached);
        editor.putInt("perfectGames", perfectGames); // Sauvegarde stat
        // averageGameTimeMs non sauvegardé
        editor.putLong("bestWinningTimeMs", bestWinningTimeMs);
        editor.putLong("worstWinningTimeMs", worstWinningTimeMs);

        editor.putInt("multiplayerGamesWon", multiplayerGamesWon);
        editor.putInt("multiplayerGamesPlayed", multiplayerGamesPlayed);
        editor.putInt("multiplayerBestWinningStreak", multiplayerBestWinningStreak);
        // multiplayerAverageScore non sauvegardé
        editor.putLong("multiplayerAverageGameTimeMs", multiplayerAverageGameTimeMs);
        editor.putInt("totalMultiplayerLosses", totalMultiplayerLosses);
        editor.putInt("multiplayerHighScore", multiplayerHighestScore);

        editor.apply();
    }

    /**
     * Formate une durée en millisecondes en une chaîne de caractères (hh:mm:ss ou mm:ss).
     */
    private String formatTime(long milliseconds) { // Nouvelle méthode
        long seconds = (milliseconds / 1000) % 60;
        long minutes = (milliseconds / (1000 * 60)) % 60;
        long hours = milliseconds / (1000 * 60 * 60);

        if (hours > 0) {
            return String.format("%02d:%02d:%02d", hours, minutes, seconds);
        } else {
            return String.format("%02d:%02d", minutes, seconds);
        }
    }

    /**
     * Affiche l'écran du menu (placeholder avec AlertDialog).
     */
    private void showMenu() { // Modifié pour utiliser AlertDialog
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle("Menu")
                .setMessage("Fonctionnalité de menu à venir !")
                .setPositiveButton("OK", (dialog, id) -> { /* Rien */ });
        AlertDialog dialog = builder.create();
        dialog.show();
    }

    /**
     * Affiche l'écran du mode multijoueur (placeholder avec AlertDialog).
     */
    private void showMultiplayerScreen() { // Modifié pour utiliser AlertDialog
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle("Multijoueur")
                .setMessage("Fonctionnalité multijoueur à venir !")
                .setPositiveButton("OK", (dialog, id) -> { /* Rien */ });
        AlertDialog dialog = builder.create();
        dialog.show();
    }

    @Override
    protected void onResume() { // Ajouté/Modifié
        super.onResume();
        // Ne redémarre le chrono que si la partie n'est pas finie
        if (game != null && !game.isGameOver() && !game.isGameWon()) {
            currentGameStartTimeMs = System.currentTimeMillis();
        }
        // Réaffiche les stats si elles étaient visibles
        if (statisticsVisible) {
            // Assure-toi que la vue est prête avant de mettre à jour
            if (findViewById(R.id.statistics_layout) != null) {
                updateStatisticsTextViews();
                findViewById(R.id.statistics_layout).setVisibility(View.VISIBLE);
                getMultiplayerButton().setVisibility(View.GONE);
            } else {
                // Si la vue n'est pas encore gonflée, on la montre (ce qui la gonflera et mettra à jour)
                showStatistics();
            }
        }
    }

    @Override
    protected void onPause() { // Modifié pour calculer temps et sauver stats
        super.onPause();
        if (game != null) {
            // Ajoute le temps écoulé seulement si la partie n'est pas finie
            if (!game.isGameOver() && !game.isGameWon()) {
                totalPlayTimeMs += System.currentTimeMillis() - currentGameStartTimeMs;
            }
            saveGame();       // Sauvegarde état jeu
            saveStatistics(); // Sauvegarde stats
        }
    }

    private void saveGame() {
        SharedPreferences.Editor editor = preferences.edit();
        editor.putString(GAME_STATE_KEY, game.toString());
        editor.putInt(HIGH_SCORE_KEY, game.getHighestScore());
        editor.apply();
    }

    /**
     * Affiche une boîte de dialogue lorsque le joueur gagne la partie (atteint 2048).
     */
    private void showGameWonDialog() { // Ajouté
        // Met à jour les stats de victoire avant d'afficher
        numberOfTimesObjectiveReached++;
        long timeTaken = System.currentTimeMillis() - currentGameStartTimeMs;
        if(timeTaken < bestWinningTimeMs){ bestWinningTimeMs = timeTaken; }
        if(timeTaken > worstWinningTimeMs){ worstWinningTimeMs = timeTaken; }

        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle("Vous avez gagné !")
                .setMessage("Félicitations ! Vous avez atteint 2048 !")
                .setPositiveButton("Nouvelle partie", (dialog, which) -> startNewGame())
                .setNegativeButton("Quitter", (dialog, which) -> finish())
                .setCancelable(false)
                .show();
    }

    /**
     * Affiche une boîte de dialogue lorsque le joueur perd la partie (aucun mouvement possible).
     */
    private void showGameOverDialog() { // Ajouté
        totalGamesPlayed++; // Met à jour stat avant d'afficher

        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle("Partie terminée !")
                .setMessage("Aucun mouvement possible. Votre score final est : " + game.getCurrentScore())
                .setPositiveButton("Nouvelle partie", (dialog, which) -> startNewGame())
                .setNegativeButton("Quitter", (dialog, which) -> finish())
                .setCancelable(false)
                .show();
    }

}